@@ -9,15 +9,14 @@ fn main() -> Result<(), Error> {
         .user("root")
         .password("root")
         .zone_id("UTC+8")
-        // .log_level("debug")
+        .debug(true)
         .build();
 
     // open session
     let mut session = Session::new(config).open()?;
     println!("time_zone: {}", session.time_zone()?);
-
-    session.sql("SHOW STORAGE GROUP")?.show();
-    // session.set_storage_group("root.ln")?;
+    session.delete_storage_group("root.ln")?;
+    session.set_storage_group("root.ln")?;
     session.create_time_series(
         "root.ln.wf01.wt01.temperature",
         DataType::INT64,
@@ -32,7 +31,14 @@ fn main() -> Result<(), Error> {
         Compressor::default(),
     )?;
 
-    session.sql("SHOW TIMESERIES root.ln")?.show();
+    session.sql("INSERT INTO root.ln.wf01.wt01(timestamp,status) values(100,true)")?;
+    session.sql("INSERT INTO root.ln.wf01.wt01(timestamp,status) values(200,false)")?;
+    session.sql(
+        "INSERT INTO root.ln.wf01.wt01(timestamp,status,temperature) values(300,false,18.36)",
+    )?;
+    session.sql(
+        "INSERT INTO root.ln.wf01.wt01(timestamp,status,temperature) values(400,true,32.23)",
+    )?;
     session.sql("select * from root.ln")?.show();
 
     session.close()?;
