@@ -22,7 +22,7 @@
 //!
 //! ```toml
 //! [dependencies]
-//! iotdb = "0.0.4"
+//! iotdb = "0.0.5"
 //! ```
 //!
 //! # Example
@@ -39,13 +39,14 @@
 //!         .user("root")
 //!         .password("root")
 //!         .zone_id("UTC+8")
-//!         // .log_level("debug")
+//!         // .debug(true)
 //!         .build();
 //!
-//! // open session
+//!     // open session
 //!     let mut session = Session::new(config).open()?;
-//!     session.sql("SHOW STORAGE GROUP")?.show();
-//!     // session.set_storage_group("root.ln")?;
+//!     println!("time_zone: {}", session.time_zone()?);
+//!     session.delete_storage_group("root.ln")?;
+//!     session.set_storage_group("root.ln")?;
 //!     session.create_time_series(
 //!         "root.ln.wf01.wt01.temperature",
 //!         DataType::INT64,
@@ -60,8 +61,15 @@
 //!         Compressor::default(),
 //!     )?;
 //!
-//!    session.sql("SHOW TIMESERIES root.ln")?.show();
-//!    session.sql("select * from root.ln")?.show();
+//!     session.sql("INSERT INTO root.ln.wf01.wt01(timestamp,status) values(100,true)")?;
+//!     session.sql("INSERT INTO root.ln.wf01.wt01(timestamp,status) values(200,false)")?;
+//!     session.sql(
+//!         "INSERT INTO root.ln.wf01.wt01(timestamp,status,temperature) values(300,false,18.36)",
+//!     )?;
+//!     session.sql(
+//!         "INSERT INTO root.ln.wf01.wt01(timestamp,status,temperature) values(400,true,32.23)",
+//!     )?;
+//!     session.sql("select * from root.ln")?.show();
 //!
 //!     session.close()?;
 //!
@@ -69,9 +77,11 @@
 //! }
 //! ```
 pub mod common;
-mod dataset;
+pub mod dataset;
 pub mod rpc;
 
+pub use thrift;
+
 use crate::common::{Compressor, DataType, Encoding, Logger};
 use crate::dataset::DataSet;
 use crate::rpc::{
@@ -85,6 +95,9 @@ use chrono::{Local, Utc};
 use log::{debug, error};
 use mimalloc::MiMalloc;
 use std::collections::BTreeMap;
+use std::env;
+use std::net::TcpStream;
+use std::str::FromStr;
 use thrift::protocol::{
     TBinaryInputProtocol, TBinaryOutputProtocol, TCompactInputProtocol, TCompactOutputProtocol,
     TInputProtocol, TOutputProtocol,
@@ -101,33 +114,77 @@ static GLOBAL: MiMalloc = MiMalloc;
 type ClientType = TSIServiceSyncClient<Box<dyn TInputProtocol>, Box<dyn TOutputProtocol>>;
 
 const SUCCESS_CODE: i32 = 200;
+const LOG_LEVER_KEY: &str = "IOTDB_LOG_LEVER";
+
+#[derive(Clone, Debug)]
+pub struct Endpoint {
+    pub host: String,
+    pub port: String,
+}
+
+impl FromStr for Endpoint {
+    type Err = ();
+
+    fn from_str(str: &str) -> Result<Self, Self::Err> {
+        let host_port: Vec<&str> = str.split(':').collect();
+        if host_port.is_empty() || host_port.len() != 2 {
+            panic!("Endpoint format error, endpoint: '{}'", str)
+        } else {
+            Ok(Self {
+                host: String::from(host_port[0]),
+                port: String::from(host_port[1]),
+            })
+        }
+    }
+}
+
+impl Default for Endpoint {
+    fn default() -> Self {
+        "127.0.0.1:6667".parse::<Self>().unwrap()
+    }
+}
+
+impl Endpoint {
+    pub fn new(host: &str, port: &str) -> Self {
+        Self {
+            host: String::from(host),
+            port: String::from(port),
+        }
+    }
+}
+
+impl ToString for Endpoint {
+    fn to_string(&self) -> String {
+        format!("{}:{}", self.host, self.port)
+    }
+}
 
 /// IotDB Config
 #[derive(Clone, Debug)]
 pub struct Config {
-    user: String,
-    password: String,
-    zone_id: String,
-    timeout: i64,
-    fetch_size: i32,
-    endpoint: String,
-    log_level: String,
-    rpc_compaction: bool,
-    protocol_version: TSProtocolVersion,
-    enable_redirect_query: bool,
-    config_map: BTreeMap<String, String>,
+    pub user: String,
+    pub password: String,
+    pub zone_id: String,
+    pub timeout: i64,
+    pub fetch_size: i32,
+    pub endpoint: Endpoint,
+    pub log_level: String,
+    pub rpc_compaction: bool,
+    pub protocol_version: TSProtocolVersion,
+    pub enable_redirect_query: bool,
+    pub config_map: BTreeMap<String, String>,
 }
 
 impl Default for Config {
     fn default() -> Self {
         Self {
-            endpoint: "127.0.0.1:6667".to_string(),
+            endpoint: Endpoint::default(),
             user: "root".to_string(),
             password: "root".to_string(),
             timeout: 3000,
             zone_id: format!("{}{}", Utc::now().offset(), Local::now().offset()),
             fetch_size: 1024,
-            log_level: "info".to_string(),
+            log_level: env::var(LOG_LEVER_KEY).unwrap_or_else(|_| "info".to_string()),
             rpc_compaction: false,
             protocol_version: TSProtocolVersion::IOTDB_SERVICE_PROTOCOL_V3,
             enable_redirect_query: false,
@@ -138,11 +195,13 @@ impl Default for Config {
 
 impl Config {
     pub fn new() -> Self {
-        Config::default()
+        let default = Config::default();
+        debug!("{:?}", default);
+        default
     }
 
     pub fn endpoint(&mut self, host: &str, port: &str) -> &mut Self {
-        self.endpoint = format!("{}:{}", host, port);
+        self.endpoint = Endpoint::new(host, port);
         self
     }
 
@@ -170,11 +229,19 @@ impl Config {
         self.fetch_size = fetch_size;
         self
     }
+
     pub fn log_level(&mut self, level: &str) -> &mut Self {
         self.log_level = level.to_string();
         self
     }
 
+    pub fn debug(&mut self, debug: bool) -> &mut Self {
+        if debug {
+            self.log_level = "debug".to_string();
+        }
+        self
+    }
+
     pub fn enable_rpc_compaction(&mut self) -> &mut Self {
         self.rpc_compaction = true;
         self
@@ -190,12 +257,12 @@ impl Config {
         self
     }
 
-    pub fn config(&mut self, key: &str, value: &str) -> &mut Self {
+    pub fn set(&mut self, key: &str, value: &str) -> &mut Self {
         self.config_map.insert(key.to_string(), value.to_string());
         self
     }
 
-    pub fn config_map(&mut self, map: &mut BTreeMap<String, String>) -> &mut Self {
+    pub fn set_map(&mut self, map: &mut BTreeMap<String, String>) -> &mut Self {
         self.config_map.append(map);
         self
     }
@@ -218,15 +285,23 @@ impl Session {
         Logger::new(config.log_level.as_str(), None).init().unwrap();
         debug!("{:?}", &config);
 
-        let mut channel = TTcpChannel::new();
-        match channel.open(config.endpoint.clone().as_str()) {
-            Ok(_) => debug!("Open a channel, endpoint: {}", config.endpoint),
-            Err(error) => debug!(
-                "Connect to endpoint: {} failed, reason: {}",
-                config.endpoint, error
-            ),
-        }
-
+        let stream: TcpStream = match TcpStream::connect(config.endpoint.to_string()) {
+            Ok(tcp_stream) => {
+                debug!("TcpStream connect to {:?}", config.endpoint);
+                tcp_stream
+            }
+            Err(error) => {
+                error!(
+                    "TcpStream connect to {:?} failed, reason: {}",
+                    config.endpoint, error
+                );
+                panic!(
+                    "TcpStream connect to {:?} failed, reason: {}",
+                    config.endpoint, error
+                );
+            }
+        };
+        let channel = TTcpChannel::with_stream(stream);
         let (channel_in, channel_out) = channel.split().unwrap();
         let (transport_in, transport_out) = (
             TFramedReadTransport::new(channel_in),
@@ -246,7 +321,7 @@ impl Session {
 
         Self {
             client: TSIServiceSyncClient::new(protocol_in, protocol_out),
-            config: config.clone(),
+            config,
             session_id: -1,
             statement_id: -1,
             is_close: true,
@@ -256,25 +331,25 @@ impl Session {
     // Open Session
     pub fn open(mut self) -> Result<Session, Error> {
         let open_req = TSOpenSessionReq::new(
-            self.config.protocol_version.clone(),
+            self.config.protocol_version,
             self.config.zone_id.clone(),
             self.config.user.clone(),
             self.config.password.clone(),
             self.config.config_map.clone(),
         );
 
-        match self.client.open_session(open_req.clone()) {
+        match self.client.open_session(open_req) {
             Ok(resp) => {
                 let status = resp.status;
                 if self.is_success(&status) {
-                    if self.config.protocol_version.clone() != resp.server_protocol_version {
+                    if self.config.protocol_version != resp.server_protocol_version {
                         self.is_close = true;
                         let msg = format!(
                             "Protocol version is different, client is {:?}, server is {:?}",
                             self.config.protocol_version.clone(),
                             resp.server_protocol_version
                         );
-                        error!("{}", msg.clone());
+                        error!("{}", msg);
                         Err(thrift::new_protocol_error(
                             ProtocolErrorKind::BadVersion,
                             msg,
@@ -284,7 +359,7 @@ impl Session {
                         self.session_id = resp.session_id.unwrap();
                         self.statement_id = self.client.request_statement_id(self.session_id)?;
                         debug!(
-                            "Open a session, session id: {}, statement id: {} ",
+                            "Open a session,session id: {}, statement id: {} ",
                             self.session_id.clone(),
                             self.statement_id.clone()
                         );
@@ -294,7 +369,7 @@ impl Session {
                     error!("{}", status.message.clone().unwrap());
                     Err(thrift::new_application_error(
                         ApplicationErrorKind::MissingResult,
-                        status.message.unwrap(),
+                        status.message.unwrap_or_else(|| "None".to_string()),
                     ))
                 }
             }
@@ -306,11 +381,11 @@ impl Session {
     }
 
     pub fn is_open(&self) -> bool {
-        !self.is_close.clone()
+        !self.is_close
     }
 
     pub fn is_close(&self) -> bool {
-        self.is_close.clone()
+        self.is_close
     }
 
     // Close Session
@@ -329,11 +404,11 @@ impl Session {
                         error!(
                             "Session closed failed, code: {}, reason: {}",
                             status.code.clone(),
-                            status.message.clone().unwrap_or("None".to_string())
+                            status.message.clone().unwrap_or_else(|| "None".to_string())
                         );
                         Err(thrift::new_application_error(
                             ApplicationErrorKind::MissingResult,
-                            status.message.clone().unwrap_or("None".to_string()),
+                            status.message.unwrap_or_else(|| "None".to_string()),
                         ))
                     }
                 }
@@ -351,16 +426,16 @@ impl Session {
             Ok(status) => {
                 if self.is_success(&status) {
                     debug!(
-                        "Set storage group {:?} message: {:?}",
+                        "Set storage group {:?}, message: {:?}",
                         storage_group,
-                        status.message.unwrap()
+                        status.message.unwrap_or_else(|| "None".to_string())
                     );
                     Ok(())
                 } else {
                     error!("{}", status.message.clone().unwrap());
                     Err(thrift::new_application_error(
                         ApplicationErrorKind::MissingResult,
-                        status.message.unwrap(),
+                        status.message.unwrap_or_else(|| "None".to_string()),
                     ))
                 }
             }
@@ -383,16 +458,16 @@ impl Session {
             Ok(status) => {
                 if self.is_success(&status) {
                     debug!(
-                        "Delete storage group(s) {:?} message: {:?}",
-                        storage_groups.clone(),
-                        status.message.unwrap()
+                        "Delete storage group(s) {:?}, message: {:?}",
+                        storage_groups,
+                        status.message.unwrap_or_else(|| "None".to_string())
                     );
                     Ok(())
                 } else {
                     error!("{}", status.message.clone().unwrap());
                     Err(thrift::new_application_error(
                         ApplicationErrorKind::MissingResult,
-                        status.message.unwrap(),
+                        status.message.unwrap_or_else(|| "None".to_string()),
                     ))
                 }
             }
@@ -429,7 +504,7 @@ impl Session {
                         Ok(status) => {
                             if self.is_success(&status) {
                                 debug!(
-                                    "Creat time series {:?} message: {:?}",
+                                    "Creat time series {:?}, message: {:?}",
                                     ts_path,
                                     status.message.unwrap()
                                 );
@@ -473,8 +548,8 @@ impl Session {
             Ok(status) => {
                 if self.is_success(&status) {
                     debug!(
-                        "Creating multiple time series {:?} message: {:?}",
-                        ts_path_vec.clone(),
+                        "Creating multiple time series {:?}, message: {:?}",
+                        ts_path_vec,
                         status.message.unwrap()
                     );
                     Ok(())
@@ -499,8 +574,8 @@ impl Session {
             Ok(status) => {
                 if self.is_success(&status) {
                     debug!(
-                        "Deleting multiple time series {:?} message: {:?}",
-                        path_vec.clone(),
+                        "Deleting multiple time series {:?}, message: {:?}",
+                        path_vec,
                         status.message.unwrap()
                     );
                     Ok(())
@@ -524,9 +599,9 @@ impl Session {
             self.session_id,
             statement,
             self.statement_id,
-            config.fetch_size.clone(),
-            config.timeout.clone(),
-            config.enable_redirect_query.clone(),
+            config.fetch_size,
+            config.timeout,
+            config.enable_redirect_query,
             false,
         );
 
@@ -556,7 +631,7 @@ impl Session {
                 if self.is_success(&status) {
                     debug!(
                         "Delete data from {:?}, message: {:?}",
-                        path_vec.clone(),
+                        path_vec,
                         status.message.unwrap()
                     );
                     Ok(())
@@ -593,8 +668,8 @@ impl Session {
             Ok(status) => {
                 if self.is_success(&status) {
                     debug!(
-                        "Insert string records to device {:?} message: {:?}",
-                        device_ids.clone(),
+                        "Insert string records to device {:?}, message: {:?}",
+                        device_ids,
                         status.message.unwrap()
                     );
                     Ok(())
@@ -631,8 +706,8 @@ impl Session {
             Ok(status) => {
                 if self.is_success(&status) {
                     debug!(
-                        "Insert one record to device {:?} message: {:?}",
-                        device_id.clone(),
+                        "Insert one record to device {:?}, message: {:?}",
+                        device_id,
                         status.message.unwrap()
                     );
                     Ok(())
@@ -652,7 +727,7 @@ impl Session {
     /// request, this method should be used to test other time cost in client
     pub fn test_insert_record(
         &mut self,
-        prefix_path: String,
+        prefix_path: &str,
         timestamp: i64,
         measurements: Vec<String>,
         values: Vec<u8>,
@@ -660,7 +735,7 @@ impl Session {
     ) -> Result<(), Error> {
         let req = TSInsertRecordReq::new(
             self.session_id,
-            prefix_path.clone(),
+            prefix_path.to_string(),
             measurements,
             values,
             timestamp,
@@ -670,8 +745,8 @@ impl Session {
             Ok(status) => {
                 if self.is_success(&status) {
                     debug!(
-                        "Testing! insert one record to prefix path {:?} message: {:?}",
-                        prefix_path.clone(),
+                        "Testing! insert one record to prefix path {:?}, message: {:?}",
+                        prefix_path,
                         status.message.unwrap()
                     );
                     Ok(())
@@ -708,8 +783,8 @@ impl Session {
             Ok(status) => {
                 if self.is_success(&status) {
                     debug!(
-                        "Insert multiple records to prefix path {:?} message: {:?}",
-                        prefix_paths.clone(),
+                        "Insert multiple records to prefix path {:?}, message: {:?}",
+                        prefix_paths,
                         status.message.unwrap()
                     );
                     Ok(())
@@ -771,11 +846,11 @@ impl Session {
     ///                  3,  688.6,  True,  text3
     /// Notice: The tablet should not have empty cell
     ///         The tablet itself is sorted
-
-    // TODO
+    /// TODO
+    #[allow(clippy::too_many_arguments)]
     pub fn insert_tablet(
         &mut self,
-        prefix_path: String,
+        prefix_path: &str,
         measurements: Vec<String>,
         values: Vec<u8>,
         timestamps: Vec<u8>,
@@ -785,7 +860,7 @@ impl Session {
     ) -> Result<TSStatus, Error> {
         let req = TSInsertTabletReq::new(
             self.session_id,
-            prefix_path,
+            prefix_path.to_string(),
             measurements,
             values,
             timestamps,
@@ -798,6 +873,7 @@ impl Session {
 
     /// insert multiple tablets, tablets are independent to each other
     /// TODO
+    #[allow(clippy::too_many_arguments)]
     pub fn insert_tablets(
         &mut self,
         prefix_paths: Vec<String>,
@@ -807,7 +883,7 @@ impl Session {
         types_list: Vec<Vec<i32>>,
         size_list: Vec<i32>,
         is_aligned: bool,
-    ) -> thrift::Result<TSStatus> {
+    ) -> Result<TSStatus, Error> {
         let req = TSInsertTabletsReq::new(
             self.session_id,
             prefix_paths,
@@ -846,72 +922,38 @@ impl Session {
     /// TODO
     pub fn gen_insert_tablets_req() {}
 
-    pub fn exec_insert(&mut self, statement: &str) -> Result<DataSet, Error> {
-        self.exec(statement)
-    }
-
     pub fn sql(&mut self, sql: &str) -> Result<DataSet, Error> {
-        self.exec_query(sql)
-    }
-
-    /// execute query sql statement and return a DataSet
-    fn exec_query(&mut self, query: &str) -> Result<DataSet, Error> {
-        debug!("Exec query \"{}\"", &query);
-        let req = TSExecuteStatementReq::new(
-            self.session_id,
-            query.to_string(),
-            self.statement_id,
-            self.config.fetch_size.clone(),
-            self.config.timeout.clone(),
-            self.config.enable_redirect_query.clone(),
-            false,
-        );
-
-        match self.client.execute_query_statement(req) {
-            Ok(resp) => {
-                if self.is_success(&resp.status) {
-                    Ok(DataSet::new(
-                        query.to_string(),
-                        self.session_id,
-                        self.config.fetch_size.clone(),
-                        resp,
-                    ))
-                } else {
-                    error!(
-                        "Exec query failed, code: {}, reason: {}",
-                        resp.status.code.clone(),
-                        resp.status.clone().message.unwrap_or("None".to_string())
-                    );
-                    Err(thrift::new_application_error(
-                        ApplicationErrorKind::MissingResult,
-                        resp.status.message.unwrap(),
-                    ))
-                }
-            }
-            Err(error) => Err(error),
-        }
+        self.exec(sql)
     }
 
     /// execute query sql statement and return a DataSet
     fn exec(&mut self, statement: &str) -> Result<DataSet, Error> {
-        debug!("Exec statement \"{}\"", &statement);
+        debug!("Exec statement \"{}\"", statement);
         let req = TSExecuteStatementReq::new(
             self.session_id,
             statement.to_string(),
-            self.statement_id.clone(),
-            self.config.fetch_size.clone(),
-            self.config.timeout.clone(),
-            self.config.enable_redirect_query.clone(),
+            self.statement_id,
+            self.config.fetch_size,
+            self.config.timeout,
+            self.config.enable_redirect_query,
             false,
         );
 
         match self.client.execute_statement(req) {
             Ok(resp) => {
                 if self.is_success(&resp.status) {
+                    debug!(
+                        "Execute statement {:?}, message: {:?}",
+                        statement,
+                        resp.status
+                            .clone()
+                            .message
+                            .unwrap_or_else(|| "None".to_string())
+                    );
                     Ok(DataSet::new(
                         statement.to_string(),
                         self.session_id,
-                        self.config.fetch_size.clone(),
+                        self.config.fetch_size,
                         resp,
                     ))
                 } else {
@@ -928,37 +970,99 @@ impl Session {
 
     /// execute batch statement and return a DataSets
     pub fn exec_batch(&mut self, statements: Vec<String>) {
-        let req = TSExecuteBatchStatementReq::new(self.session_id, statements);
+        let req = TSExecuteBatchStatementReq::new(self.session_id, statements.clone());
         match self.client.execute_batch_statement(req) {
             Ok(status) => {
-                if self.is_success(&status) {
+                if !self.is_success(&status) {
+                    debug!(
+                        "Execute statements {:?}, message: {:?}",
+                        statements,
+                        status.message.unwrap_or_else(|| "None".to_string())
+                    );
                 } else {
-                    error!("{}", status.message.clone().unwrap());
+                    error!("{}", status.message.unwrap());
                 }
             }
             Err(error) => error!("{}", error),
         }
     }
 
+    /// execute query sql statement and return a DataSet
+    pub fn exec_query(&mut self, query: &str) -> Result<DataSet, Error> {
+        debug!("Exec query \"{}\"", &query);
+        let req = TSExecuteStatementReq::new(
+            self.session_id,
+            query.to_string(),
+            self.statement_id,
+            self.config.fetch_size,
+            self.config.timeout,
+            self.config.enable_redirect_query,
+            false,
+        );
+
+        match self.client.execute_query_statement(req) {
+            Ok(resp) => {
+                if self.is_success(&resp.status) {
+                    debug!(
+                        "Execute query {:?}, message: {:?}",
+                        query,
+                        resp.status
+                            .clone()
+                            .message
+                            .unwrap_or_else(|| "None".to_string())
+                    );
+                    Ok(DataSet::new(
+                        query.to_string(),
+                        self.session_id,
+                        self.config.fetch_size,
+                        resp,
+                    ))
+                } else {
+                    error!(
+                        "Exec query failed, code: {}, reason: {}",
+                        resp.status.code.clone(),
+                        resp.status
+                            .clone()
+                            .message
+                            .unwrap_or_else(|| "None".to_string())
+                    );
+                    Err(thrift::new_application_error(
+                        ApplicationErrorKind::MissingResult,
+                        resp.status.message.unwrap(),
+                    ))
+                }
+            }
+            Err(error) => Err(error),
+        }
+    }
+
     /// execute update statement and return a DataSet
     pub fn exec_update(&mut self, statement: &str) -> Result<DataSet, Error> {
         let req = TSExecuteStatementReq::new(
             self.session_id,
             statement.to_string(),
             self.statement_id,
-            self.config.fetch_size.clone(),
-            self.config.timeout.clone(),
-            self.config.enable_redirect_query.clone(),
+            self.config.fetch_size,
+            self.config.timeout,
+            self.config.enable_redirect_query,
             false,
         );
 
         match self.client.execute_update_statement(req) {
             Ok(resp) => {
                 if self.is_success(&resp.status) {
+                    debug!(
+                        "Execute update statement {:?}, message: {:?}",
+                        statement,
+                        resp.status
+                            .clone()
+                            .message
+                            .unwrap_or_else(|| "None".to_string())
+                    );
                     Ok(DataSet::new(
                         statement.to_string(),
                         self.session_id,
-                        self.config.fetch_size.clone(),
+                        self.config.fetch_size,
                         resp,
                     ))
                 } else {
@@ -983,11 +1087,11 @@ impl Session {
         let req = TSRawDataQueryReq::new(
             self.session_id,
             paths,
-            self.config.fetch_size.clone(),
+            self.config.fetch_size,
             start_time,
             end_time,
             self.statement_id,
-            self.config.enable_redirect_query.clone(),
+            self.config.enable_redirect_query,
             false,
         );
 
@@ -997,7 +1101,7 @@ impl Session {
                     Ok(DataSet::new(
                         "".to_string(),
                         self.session_id,
-                        self.config.fetch_size.clone(),
+                        self.config.fetch_size,
                         resp,
                     ))
                 } else {
@@ -1033,7 +1137,7 @@ impl Session {
 
     /// Get time zone
     pub fn time_zone(&mut self) -> Result<String, Error> {
-        match self.client.get_time_zone(self.session_id.clone()) {
+        match self.client.get_time_zone(self.session_id) {
             Ok(resp) => {
                 if resp.status.code == 200 {
                     Ok(resp.time_zone)
@@ -1051,11 +1155,8 @@ impl Session {
         status.code == SUCCESS_CODE
     }
 
-    fn check_sorted(timestamps: Vec<i64>) {
-        todo!()
-    }
-
     /// Cancel operation
+    #[allow(dead_code)]
     fn cancel_operation(&mut self, query_id: i64) -> Result<(), Error> {
         let req = TSCancelOperationReq::new(self.session_id, query_id);
         match self.client.cancel_operation(req) {
@@ -1064,7 +1165,7 @@ impl Session {
                     Ok(())
                 } else {
                     let msg = format!("Cancel operation failed,'{:?}'", query_id);
-                    error!("{}", msg.clone());
+                    error!("{}", msg);
                     Err(thrift::new_application_error(
                         ApplicationErrorKind::MissingResult,
                         msg,
